<?php

class connectDB
{
  private $_tables = array(
    'attachment',
    'attachment_lang',
    'category',
    'category_lang',
    'category_shop',
    'category_group',
    'product',
    'product_shop',
    'feature',
    'feature_lang',
    'feature_shop',
    'feature_value',
    'feature_value_lang',
    'feature_product',
    'product_lang',
    'category_product',
    'product_tag',
    'image',
    'image_lang',
    'image_shop',
    'image_type',
    'specific_price',
    'specific_price_priority',
    'specific_price_rule',
    'specific_price_rule_condition',
    'specific_price_rule_condition_group',
    'product_carrier',
    'cart_product',
    'product_attachment',
    'product_country_tax',
    'product_download',
    'product_group_reduction_cache',
    'product_sale',
    'product_supplier',
    'warehouse_product_location',
    'stock',
    'stock_available',
    'stock_mvt',
    'customization',
    'customization_field',
    'supply_order_detail',
    'attribute_impact',
    'pack',
    'attribute',
    'attribute_lang',
    'attribute_group',
    'attribute_group_lang',
    'attribute_group_shop',
    'attribute_shop',
    'product_attribute',
    'product_attribute_shop',
    'product_attribute_combination',
    'product_attribute_image',
    'address',
    'manufacturer',
    'manufacturer_lang',
    'manufacturer_shop',
    'supplier',
    'supplier_lang',
    'supplier_shop',
    'search_engine',
    'search_word',
    'search_index',
    'configuration',
    'shop',
    'shop_group',
    'shop_url',
    'configuration_lang',
    'lang',
    'lang_shop',
    'country',
    'country_lang',
    'country_shop',
    'hook',
    'module',
    'module_shop',
    'module_group',
    'hook_module',
    'hook_alias',
    'currency',
    'currency_lang',
    'currency_shop',
    'group',
    'group_lang',
    'group_shop',
    'zone',
    'state',
    'tax_rule',
    'tax',
    'tax_lang',
    'tax_rules_group',
    'tax_rules_group_shop',
    'tag',
    'tag_count',
    'smarty_lazy_cache',
  );

  public function backupTables()
  {
    $response = array();
    $response['skip_lock'] = 0;
    $pass = _DB_PASSWD_;
    $user = _DB_USER_;
    $host = _DB_SERVER_;
    $file = dirname(__FILE__) . '/../files/dump.sql';
    $tablesList = $this->_getTablesList();
    $command = " mysqldump -h $host -u $user -p'{$pass}' "._DB_NAME_." $tablesList > $file";
    exec($command, $output, $res);
    if( $res ){
      $response['skip_lock'] = 1;
      $command = " mysqldump --single-transaction -h $host -u $user -p'{$pass}' "._DB_NAME_." $tablesList > $file";
      exec($command, $output, $res);
    }

    if( $res ){
      throw new Exception('Can not get your DB');
    }
    $this->_zipFile($file);
    $this->_backupDb();

    return $response;
  }

  private function _backupDb()
  {
    $backupPath = dirname(__FILE__) . '/../db_backups/';
    $archive = dirname(__FILE__) . '/../files/dump.zip';
    if( !file_exists( $backupPath . date('d-m-Y') . '.zip' ) ){
      @copy($archive, $backupPath . date('d-m-Y') . '.zip');
    }
    $this->_clearOldBackups();
  }

  public function _clearOldBackups()
  {
    clearstatcache();
    $path = dirname(__FILE__) . '/../db_backups/';
    if( file_exists($path) ){
      foreach (scandir($path) as $d) {
        if ( is_file($path.$d) && strpos($path.$d, 'zip') !== false ) {
          $backupTime = filemtime($path.$d);
          $backupTime = strtotime('+7 day',$backupTime);
          if( time() > $backupTime ){
            unlink($path.$d);
          }
        }
      }
    }
  }

  private function _zipFile($file)
  {
    $archive = dirname(__FILE__) . '/../files/dump.zip';
    if( file_exists($archive) ){
      unlink($archive);
    }
    $zip = new ZipArchive;
    if ($zip->open($archive, ZipArchive::CREATE) === TRUE)
    {
      $zip->addFile($file,'dump.sql');
      $zip->close();
    }
    else{
      throw new Exception('Can not create archive');
    }
  }

  private function _getTablesList()
  {
    $sql = "SHOW TABLES";
    $res = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql);
    foreach (  $res as $key=>$table ){
      foreach ( $table as $t ){
        $res[$key] = $t;
      }
    }
    foreach ( $this->_tables as $key=>$table ){
      if( !in_array(_DB_PREFIX_.$table, $res) ){
        unset($this->_tables[$key]);
      }
    }

    $tablesList = '';
    foreach ( $this->_tables as $table ){
      $tablesList .= _DB_PREFIX_ . $table. ' ';
    }

    return $tablesList;
  }

  public function restoreDump()
  {
    $this->_unzipDump();
    $this->_restoreDB();
    $this->_removeOldImages();
    $this->_removeAttributeColorImages();
    $this->_copyDownloads();
    $this->_copyColors();
    $dumps = dirname(__FILE__) . '/../files/dumps/';
    $this->_cleanZipFolder($dumps);
  }

  private function _removeAttributeColorImages()
  {
    $sql = "
      SELECT id_attribute
      FROM " . _DB_PREFIX_ . "sync_master_simpleimport_images
      WHERE id_attribute IS NOT NULL
    ";

    $res = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql);

    foreach ( $res as $image ){
      if( file_exists(_PS_COL_IMG_DIR_ . $image['id_attribute'] . '.jpg') ){
        unlink(_PS_COL_IMG_DIR_ . $image['id_attribute'] . '.jpg');
      }
    }
  }

  private function _removeOldImages()
  {
    $sql = "
      SELECT id_image
      FROM " . _DB_PREFIX_ . "sync_master_simpleimport_images
      WHERE id_attribute IS NULL
    ";

    $res = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql);

    foreach ( $res as $image ){
      $img = new Image();
      $img->id = $image['id_image'];
      $img->deleteImage();
    }
  }

  private function _restoreDB()
  {
    $db = dirname(__FILE__) . '/../files/dumps/dump.sql';
    if( !file_exists($db) ){
      throw new Exception('Dump not found!');
    }

    $pass = _DB_PASSWD_;
    $user = _DB_USER_;
    $dbName = _DB_NAME_;
    $host = _DB_SERVER_;
    $command = " mysql -h $host -u $user -p'{$pass}' $dbName < $db ";
    exec($command, $output, $res);
    if( $res ){
      throw new Exception('Can not restore DB');
    }
  }

  private function _unzipDump()
  {
    $path = dirname(__FILE__) . '/../files/';
    $file = $path.'dump.zip';
    $zip = new ZipArchive;
    $res = $zip->open($file);
    if ($res === TRUE) {
      $this->_cleanZipFolder($path . 'dumps/');
      $zip->extractTo($path);
      $zip->close();
    }
    else {
      throw new Exception( 'Can not unzip your dump!' );
    }
  }

  protected function _cleanZipFolder( $path, $removeDir = false )
  {
    if( file_exists($path) ){
      foreach (scandir($path) as $d) {
        if ( is_file($path.$d) ) {
          unlink($path.$d);
        }
        elseif( is_dir( $path.$d ) ){
          if($d == '.' || $d == '..'){
            continue;
          }
          $this->_cleanZipFolder($path.$d . '/', true);
        }
      }

      if( $removeDir ){
        rmdir($path);
      }
    }
  }

  private function _copyColors()
  {
    $path = dirname(__FILE__) . '/../files/dumps/co/';
    if( file_exists( $path  ) ){
      $this->recurseCopy($path, _PS_COL_IMG_DIR_);
    }
  }

  private function _copyDownloads()
  {
    $path = dirname(__FILE__) . '/../files/dumps/download/';
    if( file_exists( $path  ) ){
      $this->recurseCopy($path, _PS_DOWNLOAD_DIR_);
    }
  }

  private function recurseCopy(
    string $sourceDirectory,
    string $destinationDirectory,
    string $childFolder = ''
  ): void {
    $directory = opendir($sourceDirectory);

    if (is_dir($destinationDirectory) === false) {
      mkdir($destinationDirectory);
    }

    if ($childFolder !== '') {
      if (is_dir("$destinationDirectory/$childFolder") === false) {
        mkdir("$destinationDirectory/$childFolder");
      }

      while (($file = readdir($directory)) !== false) {
        if ($file === '.' || $file === '..') {
          continue;
        }

        if (is_dir("$sourceDirectory/$file") === true) {
          $this->recurseCopy("$sourceDirectory/$file", "$destinationDirectory/$childFolder/$file");
        } else {
          rename("$sourceDirectory/$file", "$destinationDirectory/$childFolder/$file");
        }
      }

      closedir($directory);

      return;
    }

    while (($file = readdir($directory)) !== false) {
      if ($file === '.' || $file === '..') {
        continue;
      }

      if (is_dir("$sourceDirectory/$file") === true) {
        $this->recurseCopy("$sourceDirectory/$file", "$destinationDirectory/$file");
      }
      else {
        rename("$sourceDirectory/$file", "$destinationDirectory/$file");
      }
    }

    closedir($directory);
  }
}